@@ -386,16 +386,16 @@ _vte_ring_discard_one_row (VteRing *ring)
 	ring->start++;
 	if (G_UNLIKELY (ring->start == ring->writable)) {
 		_vte_ring_reset_streams (ring, ring->writable);
-	} else {
+	} else if (ring->start < ring->writable) {
 		VteRowRecord record;
 		_vte_stream_advance_tail (ring->row_stream, ring->start * sizeof (record));
 		if (G_LIKELY (_vte_ring_read_row_record (ring, &record, ring->start))) {
 			_vte_stream_advance_tail (ring->text_stream, record.text_start_offset);
 			_vte_stream_advance_tail (ring->attr_stream, record.attr_start_offset);
 		}
-	}
-	if (ring->start > ring->writable)
+	} else {
 		ring->writable = ring->start;
+	}
 }
 
 static void
@@ -814,7 +814,7 @@ _vte_ring_rewrap (VteRing *ring,
 			goto err;
 		new_markers[i].row = new_markers[i].col = -1;
 		_vte_debug_print(VTE_DEBUG_RING,
-				"Marker #%d old coords:  row %ld  col %ld  ->  text_offset %ld  fragment_cells %d  eol_cells %d\n",
+				"Marker #%d old coords:  row %ld  col %ld  ->  text_offset %" G_GSIZE_FORMAT " fragment_cells %d  eol_cells %d\n",
 				i, markers[i]->row, markers[i]->col, marker_text_offsets[i].text_offset,
 				marker_text_offsets[i].fragment_cells, marker_text_offsets[i].eol_cells);
 	}
@@ -843,7 +843,7 @@ _vte_ring_rewrap (VteRing *ring,
 		VteRowRecord new_record;
 		glong col = 0;
 		_vte_debug_print(VTE_DEBUG_RING,
-				"  Old paragraph:  row %ld  (text_offset %ld)  up to (exclusive)  ",  /* no '\n' */
+				"  Old paragraph:  row %" G_GSIZE_FORMAT "  (text_offset %" G_GSIZE_FORMAT ")  up to (exclusive)  ",  /* no '\n' */
 				paragraph_start_row, paragraph_start_text_offset);
 		while (old_row_index <= ring->end) {
 			prev_record_was_soft_wrapped = old_record.soft_wrapped;
@@ -864,7 +864,7 @@ _vte_ring_rewrap (VteRing *ring,
 		if (!prev_record_was_soft_wrapped)  /* The last paragraph can be soft wrapped! */
 			paragraph_len--;  /* Strip trailing '\n' */
 		_vte_debug_print(VTE_DEBUG_RING,
-				"row %ld  (text_offset %ld)%s  len %ld  is_ascii %d\n",
+				"row %" G_GSIZE_FORMAT "  (text_offset %" G_GSIZE_FORMAT ")%s  len %" G_GSIZE_FORMAT "  is_ascii %d\n",
 				paragraph_end_row, paragraph_end_text_offset,
 				prev_record_was_soft_wrapped ? "  soft_wrapped" : "",
 				paragraph_len, paragraph_is_ascii);
@@ -907,7 +907,7 @@ _vte_ring_rewrap (VteRing *ring,
 						new_record.soft_wrapped = 1;
 						_vte_stream_append(new_row_stream, (const char *) &new_record, sizeof (new_record));
 						_vte_debug_print(VTE_DEBUG_RING,
-								"    New row %ld  text_offset %ld  attr_offset %ld  soft_wrapped\n",
+								"    New row %ld  text_offset %" G_GSIZE_FORMAT "  attr_offset %" G_GSIZE_FORMAT "  soft_wrapped\n",
 								new_row_index,
 								new_record.text_start_offset, new_record.attr_start_offset);
 						for (i = 0; i < num_markers; i++) {
@@ -954,7 +954,7 @@ _vte_ring_rewrap (VteRing *ring,
 		new_record.soft_wrapped = prev_record_was_soft_wrapped;
 		_vte_stream_append(new_row_stream, (const char *) &new_record, sizeof (new_record));
 		_vte_debug_print(VTE_DEBUG_RING,
-				"    New row %ld  text_offset %ld  attr_offset %ld\n",
+				"    New row %ld  text_offset %" G_GSIZE_FORMAT "  attr_offset %" G_GSIZE_FORMAT "\n",
 				new_row_index,
 				new_record.text_start_offset, new_record.attr_start_offset);
 		for (i = 0; i < num_markers; i++) {
@@ -988,7 +988,7 @@ _vte_ring_rewrap (VteRing *ring,
 		if (!_vte_frozen_row_text_offset_to_column(ring, new_markers[i].row, &marker_text_offsets[i], &new_markers[i].col))
 			goto err;
 		_vte_debug_print(VTE_DEBUG_RING,
-				"Marker #%d new coords:  text_offset %ld  fragment_cells %d  eol_cells %d  ->  row %ld  col %ld\n",
+				"Marker #%d new coords:  text_offset %" G_GSIZE_FORMAT "  fragment_cells %d  eol_cells %d  ->  row %ld  col %ld\n",
 				i, marker_text_offsets[i].text_offset, marker_text_offsets[i].fragment_cells,
 				marker_text_offsets[i].eol_cells, new_markers[i].row, new_markers[i].col);
 		markers[i]->row = new_markers[i].row;
@@ -1017,7 +1017,7 @@ static gboolean
 _vte_ring_write_row (VteRing *ring,
 		     GOutputStream *stream,
 		     VteRowData *row,
-		     VteTerminalWriteFlags flags,
+		     VteWriteFlags flags,
 		     GCancellable *cancellable,
 		     GError **error)
 {
@@ -1043,7 +1043,7 @@ _vte_ring_write_row (VteRing *ring,
  * _vte_ring_write_contents:
  * @ring: a #VteRing
  * @stream: a #GOutputStream to write to
- * @flags: a set of #VteTerminalWriteFlags
+ * @flags: a set of #VteWriteFlags
  * @cancellable: optional #GCancellable object, %NULL to ignore
  * @error: a #GError location to store the error occuring, or %NULL to ignore
  *
@@ -1054,7 +1054,7 @@ _vte_ring_write_row (VteRing *ring,
 gboolean
 _vte_ring_write_contents (VteRing *ring,
 			  GOutputStream *stream,
-			  VteTerminalWriteFlags flags,
+			  VteWriteFlags flags,
 			  GCancellable *cancellable,
 			  GError **error)
 {
