@@ -108,6 +108,7 @@ G_BEGIN_DECLS
 #define VTE_UPDATE_REPEAT_TIMEOUT	30
 #define VTE_MAX_PROCESS_TIME		100
 #define VTE_CELL_BBOX_SLACK		1
+#define VTE_DEFAULT_UTF8_AMBIGUOUS_WIDTH 1
 
 #define VTE_UTF8_BPC                    (6) /* Maximum number of bytes used per UTF-8 character */
 
@@ -149,6 +150,12 @@ struct vte_match_regex {
         } cursor;
 };
 
+typedef enum _VteCharacterReplacement {
+        VTE_CHARACTER_REPLACEMENT_NONE,
+        VTE_CHARACTER_REPLACEMENT_LINE_DRAWING,
+        VTE_CHARACTER_REPLACEMENT_BRITISH
+} VteCharacterReplacement;
+
 /* The terminal's keypad/cursor state.  A terminal can either be using the
  * normal keypad, or the "application" keypad. */
 typedef enum _VteKeymode {
@@ -165,6 +172,29 @@ typedef struct _VtePaletteColor {
 	} sources[2];
 } VtePaletteColor;
 
+/* These correspond to the parameters for DECSCUSR (Set cursor style). */
+typedef enum _VteCursorStyle {
+        /* We treat 0 and 1 differently, assuming that the VT510 does so too.
+         *
+         * See, according to the "VT510 Video Terminal Programmer Information",
+         * from vt100.net, paragraph "2.5.7 Cursor Display", there was a menu
+         * item in the "Terminal Set-Up" to set the cursor's style. It looks
+         * like that defaulted to blinking block. So it makes sense for 0 to
+         * mean "set cursor style to default (set by Set-Up)" and 1 to mean
+         * "set cursor style to blinking block", since that default need not be
+         * blinking block. Access to a VT510 is needed to test this theory,
+         * but it seems plausible. And, anyhow, we can even decide we know
+         * better than the VT510 designers! */
+        VTE_CURSOR_STYLE_TERMINAL_DEFAULT = 0,
+        VTE_CURSOR_STYLE_BLINK_BLOCK      = 1,
+        VTE_CURSOR_STYLE_STEADY_BLOCK     = 2,
+        VTE_CURSOR_STYLE_BLINK_UNDERLINE  = 3,
+        VTE_CURSOR_STYLE_STEADY_UNDERLINE = 4,
+        /* *_IBEAM are xterm extensions */
+        VTE_CURSOR_STYLE_BLINK_IBEAM      = 5,
+        VTE_CURSOR_STYLE_STEADY_IBEAM     = 6
+} VteCursorStyle;
+
 /* Terminal private data. */
 struct _VteTerminalPrivate {
         /* Metric and sizing data: dimensions of the window */
@@ -190,7 +220,7 @@ struct _VteTerminalPrivate {
 
 	/* Input data queues. */
 	const char *encoding;		/* the pty's encoding */
-        int iso2022_utf8_ambiguous_width;
+        int utf8_ambiguous_width;
 	struct _vte_iso2022_state *iso2022;
 	struct _vte_incoming_chunk{
 		struct _vte_incoming_chunk *next;
@@ -216,35 +246,45 @@ struct _VteTerminalPrivate {
 	 * screen, which seems to be a DEC-specific feature. */
 	struct _VteScreen {
 		VteRing row_data[1];	/* buffer contents */
-		VteVisualPosition cursor_current, cursor_saved;
-					/* the current and saved positions of
-					   the [insertion] cursor -- current is
-					   absolute, saved is relative to the
-					   insertion delta */
-		gboolean reverse_mode;	/* reverse mode */
-		gboolean origin_mode;	/* origin mode */
-		gboolean sendrecv_mode;	/* sendrecv mode */
-		gboolean insert_mode;	/* insert mode */
-		gboolean linefeed_mode;	/* linefeed mode */
-		struct vte_scrolling_region {
-			int start, end;
-		} scrolling_region;	/* the region we scroll in */
-		gboolean scrolling_restricted;
 		long scroll_delta;	/* scroll offset */
 		long insert_delta;	/* insertion offset */
-		VteCell defaults;	/* default characteristics
-					   for insertion of any new
-					   characters */
-		VteCell color_defaults;	/* original defaults
-					   plus the current
-					   fore/back */
-		VteCell fill_defaults;	/* original defaults
-					   plus the current
-					   fore/back with no
-					   character data */
-		gboolean alternate_charset;
+
+                /* Stuff saved along with the cursor */
+                struct {
+                        VteVisualPosition cursor;
+                        gboolean reverse_mode;
+                        gboolean origin_mode;
+                        gboolean sendrecv_mode;
+                        gboolean insert_mode;
+                        gboolean linefeed_mode;
+                        VteCell defaults;
+                        VteCell color_defaults;
+                        VteCell fill_defaults;
+                        VteCharacterReplacement character_replacements[2];
+                        VteCharacterReplacement *character_replacement;
+                } saved;
 	} normal_screen, alternate_screen, *screen;
 
+        /* Values we save along with the cursor */
+        VteVisualPosition cursor;	/* relative to the insertion delta */
+        gboolean reverse_mode;	/* reverse mode */
+        gboolean origin_mode;	/* origin mode */
+        gboolean sendrecv_mode;	/* sendrecv mode */
+        gboolean insert_mode;	/* insert mode */
+        gboolean linefeed_mode;	/* linefeed mode */
+        VteCell defaults;	/* default characteristics
+                                   for insertion of any new
+                                   characters */
+        VteCell color_defaults;	/* original defaults
+                                   plus the current
+                                   fore/back */
+        VteCell fill_defaults;	/* original defaults
+                                   plus the current
+                                   fore/back with no
+                                   character data */
+        VteCharacterReplacement character_replacements[2];  /* charsets in the G0 and G1 slots */
+        VteCharacterReplacement *character_replacement;     /* pointer to the active one */
+
 	/* Selection information. */
 	gboolean has_selection;
 	gboolean selecting;
@@ -270,7 +310,6 @@ struct _VteTerminalPrivate {
 	gboolean margin_bell;
 	guint bell_margin;
 	gboolean allow_bold;
-	gboolean nrc_mode;
         gboolean deccolm_mode; /* DECCOLM allowed */
 	GHashTable *tabstops;
 	gboolean text_modified_flag;
@@ -286,11 +325,17 @@ struct _VteTerminalPrivate {
 	gboolean alternate_screen_scroll;
 	long scrollback_lines;
 
-	/* Cursor shape */
+        /* Restricted scrolling */
+        struct vte_scrolling_region {
+                int start, end;
+        } scrolling_region;     /* the region we scroll in */
+        gboolean scrolling_restricted;
+
+	/* Cursor shape, as set via API */
 	VteCursorShape cursor_shape;
         float cursor_aspect_ratio;
 
-	/* Cursor blinking. */
+	/* Cursor blinking, as set in dconf. */
         VteCursorBlinkMode cursor_blink_mode;
 	gboolean cursor_blink_state;
 	guint cursor_blink_tag;           /* cursor blinking timeout ID */
@@ -301,6 +346,10 @@ struct _VteTerminalPrivate {
 	gboolean cursor_visible;
 	gboolean has_focus;               /* is the terminal window focused */
 
+        /* DECSCUSR cursor style (shape and blinking possibly overridden
+         * via escape sequence) */
+        VteCursorStyle cursor_style;
+
 	/* Input device options. */
         gboolean input_enabled;
 	time_t last_keypress_time;
@@ -450,6 +499,8 @@ VteRowData *_vte_terminal_ring_insert (VteTerminal *terminal, glong position, gb
 VteRowData *_vte_terminal_ring_append (VteTerminal *terminal, gboolean fill);
 void _vte_terminal_ring_remove (VteTerminal *terminal, glong position);
 
+void _vte_terminal_set_cursor_style(VteTerminal *terminal, VteCursorStyle style);
+
 /* vteseq.c: */
 void _vte_terminal_handle_sequence(VteTerminal *terminal,
 				   const char *match,
